@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
 # Copyright 2011 Grid Dynamics
 # Copyright 2011 Eldar Nugaev, Kirill Shileev, Ilya Alekseyev
@@ -27,7 +27,7 @@ from nova.compute import utils as compute_utils
 from nova import exception
 from nova import network
 from nova.openstack.common import log as logging
-from nova import utils
+from nova.openstack.common import uuidutils
 
 
 LOG = logging.getLogger(__name__)
@@ -67,11 +67,11 @@ def _translate_floating_ip_view(floating_ip):
     }
     try:
         result['fixed_ip'] = floating_ip['fixed_ip']['address']
-    except (TypeError, KeyError):
+    except (TypeError, KeyError, AttributeError):
         result['fixed_ip'] = None
     try:
         result['instance_id'] = floating_ip['instance']['uuid']
-    except (TypeError, KeyError):
+    except (TypeError, KeyError, AttributeError):
         result['instance_id'] = None
     return {'floating_ip': result}
 
@@ -92,10 +92,10 @@ def disassociate_floating_ip(self, context, instance, address):
     try:
         self.network_api.disassociate_floating_ip(context, instance, address)
     except exception.NotAuthorized:
-        raise webob.exc.HTTPUnauthorized()
-    except exception.FloatingIpNotAssociated:
-        msg = _('Floating ip is not associated')
-        raise webob.exc.HTTPBadRequest(explanation=msg)
+        raise webob.exc.HTTPForbidden()
+    except exception.CannotDisassociateAutoAssignedFloatingIP:
+        msg = _('Cannot disassociate auto assigned floating ip')
+        raise webob.exc.HTTPForbidden(explanation=msg)
 
 
 class FloatingIPController(object):
@@ -106,33 +106,14 @@ class FloatingIPController(object):
         self.network_api = network.API()
         super(FloatingIPController, self).__init__()
 
-    def _get_fixed_ip(self, context, fixed_ip_id):
-        if fixed_ip_id is None:
-            return None
-        try:
-            return self.network_api.get_fixed_ip(context, fixed_ip_id)
-        except exception.FixedIpNotFound:
-            return None
-
-    def _get_instance(self, context, instance_id):
-        return self.compute_api.get(context, instance_id)
-
-    def _set_metadata(self, context, floating_ip):
-        # When Quantum v2 API is used, 'fixed_ip' and 'instance' are
-        # already set. In this case we don't need to update the fields.
-
-        if 'fixed_ip' not in floating_ip:
-            fixed_ip_id = floating_ip['fixed_ip_id']
-            floating_ip['fixed_ip'] = self._get_fixed_ip(context,
-                                                         fixed_ip_id)
+    def _normalize_ip(self, floating_ip):
+        # NOTE(vish): translate expects instance to be in the floating_ip
+        #             dict but it is returned in the fixed_ip dict by
+        #             nova-network
+        fixed_ip = floating_ip.get('fixed_ip')
         if 'instance' not in floating_ip:
-            instance_uuid = None
-            if floating_ip['fixed_ip']:
-                instance_uuid = floating_ip['fixed_ip']['instance_uuid']
-
-            if instance_uuid:
-                floating_ip['instance'] = self._get_instance(context,
-                                                             instance_uuid)
+            if fixed_ip:
+                floating_ip['instance'] = fixed_ip['instance']
             else:
                 floating_ip['instance'] = None
 
@@ -145,9 +126,10 @@ class FloatingIPController(object):
         try:
             floating_ip = self.network_api.get_floating_ip(context, id)
         except exception.NotFound:
-            raise webob.exc.HTTPNotFound()
+            msg = _("Floating ip not found for id %s") % id
+            raise webob.exc.HTTPNotFound(explanation=msg)
 
-        self._set_metadata(context, floating_ip)
+        self._normalize_ip(floating_ip)
 
         return _translate_floating_ip_view(floating_ip)
 
@@ -160,7 +142,7 @@ class FloatingIPController(object):
         floating_ips = self.network_api.get_floating_ips_by_project(context)
 
         for floating_ip in floating_ips:
-            self._set_metadata(context, floating_ip)
+            self._normalize_ip(floating_ip)
 
         return _translate_floating_ips_view(floating_ips)
 
@@ -180,7 +162,7 @@ class FloatingIPController(object):
                 nmfi.message = _("No more floating ips in pool %s.") % pool
             else:
                 nmfi.message = _("No more floating ips available.")
-            raise nmfi
+            raise
 
         return _translate_floating_ip_view(ip)
 
@@ -189,7 +171,11 @@ class FloatingIPController(object):
         authorize(context)
 
         # get the floating ip object
-        floating_ip = self.network_api.get_floating_ip(context, id)
+        try:
+            floating_ip = self.network_api.get_floating_ip(context, id)
+        except exception.NotFound:
+            msg = _("Floating ip not found for id %s") % id
+            raise webob.exc.HTTPNotFound(explanation=msg)
         address = floating_ip['address']
 
         # get the associated instance object (if any)
@@ -197,7 +183,10 @@ class FloatingIPController(object):
 
         # disassociate if associated
         if floating_ip.get('fixed_ip_id'):
-            disassociate_floating_ip(self, context, instance, address)
+            try:
+                disassociate_floating_ip(self, context, instance, address)
+            except exception.FloatingIpNotAssociated:
+                LOG.info(_("Floating ip %s has been disassociated") % address)
 
         # release ip from project
         self.network_api.release_floating_ip(context, address)
@@ -259,6 +248,10 @@ class FloatingIPActionController(wsgi.Controller):
         except exception.NoFloatingIpInterface:
             msg = _('l3driver call to add floating ip failed')
             raise webob.exc.HTTPBadRequest(explanation=msg)
+        except (exception.FloatingIpNotFoundForAddress,
+                exception.NotAuthorized):
+            msg = _('floating ip not found')
+            raise webob.exc.HTTPNotFound(explanation=msg)
         except Exception:
             msg = _('Error. Unable to associate floating ip')
             LOG.exception(msg)
@@ -282,25 +275,36 @@ class FloatingIPActionController(wsgi.Controller):
             raise webob.exc.HTTPBadRequest(explanation=msg)
 
         # get the floating ip object
-        floating_ip = self.network_api.get_floating_ip_by_address(context,
-                                                                  address)
+        try:
+            floating_ip = self.network_api.get_floating_ip_by_address(context,
+                                                                      address)
+        except exception.FloatingIpNotFoundForAddress:
+            msg = _("floating ip not found")
+            raise webob.exc.HTTPNotFound(explanation=msg)
+
         # get the associated instance object (if any)
         instance = get_instance_by_floating_ip_addr(self, context, address)
 
         # disassociate if associated
         if (instance and
             floating_ip.get('fixed_ip_id') and
-            (utils.is_uuid_like(id) and
+            (uuidutils.is_uuid_like(id) and
              [instance['uuid'] == id] or
              [instance['id'] == id])[0]):
-            disassociate_floating_ip(self, context, instance, address)
+            try:
+                disassociate_floating_ip(self, context, instance, address)
+            except exception.FloatingIpNotAssociated:
+                msg = _('Floating ip is not associated')
+                raise webob.exc.HTTPBadRequest(explanation=msg)
             return webob.Response(status_int=202)
         else:
-            return webob.Response(status_int=404)
+            msg = _("Floating ip %(address)s is not associated with instance "
+                    "%(id)s.") % locals()
+            raise webob.exc.HTTPUnprocessableEntity(explanation=msg)
 
 
 class Floating_ips(extensions.ExtensionDescriptor):
-    """Floating IPs support"""
+    """Floating IPs support."""
 
     name = "FloatingIps"
     alias = "os-floating-ips"
