@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright 2010-2011 OpenStack LLC.
+# Copyright 2010-2011 OpenStack Foundation
 # Copyright 2011 Piston Cloud Computing, Inc.
 # All Rights Reserved.
 #
@@ -22,6 +22,7 @@ from nova.api.openstack import common
 from nova.api.openstack.compute.views import addresses as views_addresses
 from nova.api.openstack.compute.views import flavors as views_flavors
 from nova.api.openstack.compute.views import images as views_images
+from nova.compute import instance_types
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
 
@@ -164,20 +165,23 @@ class ViewBuilder(common.ViewBuilder):
 
     def _get_image(self, request, instance):
         image_ref = instance["image_ref"]
-        image_id = str(common.get_id_from_href(image_ref))
-        bookmark = self._image_builder._get_bookmark_link(request,
-                                                          image_id,
-                                                          "images")
-        return {
-            "id": image_id,
-            "links": [{
-                "rel": "bookmark",
-                "href": bookmark,
-            }],
-        }
+        if image_ref:
+            image_id = str(common.get_id_from_href(image_ref))
+            bookmark = self._image_builder._get_bookmark_link(request,
+                                                              image_id,
+                                                              "images")
+            return {
+                "id": image_id,
+                "links": [{
+                    "rel": "bookmark",
+                    "href": bookmark,
+                }],
+            }
+        else:
+            return ""
 
     def _get_flavor(self, request, instance):
-        instance_type = instance["instance_type"]
+        instance_type = instance_types.extract_instance_type(instance)
         if not instance_type:
             LOG.warn(_("Instance has had its instance_type removed "
                     "from the DB"), instance=instance)
@@ -208,9 +212,9 @@ class ViewBuilder(common.ViewBuilder):
 
         if fault.get('details', None):
             is_admin = False
-            context = getattr(request, 'context', None)
+            context = request.environ["nova.context"]
             if context:
-                is_admin = getattr(request.context, 'is_admin', False)
+                is_admin = getattr(context, 'is_admin', False)
 
             if is_admin or fault['code'] != 500:
                 fault_dict['details'] = fault["details"]
