@@ -24,17 +24,25 @@ Includes root and intermediate CAs, SSH key_pairs and x509 certificates.
 
 from __future__ import absolute_import
 
+import base64
 import hashlib
 import os
+import re
 import string
+import struct
+
+from oslo.config import cfg
+from pyasn1.codec.der import encoder as der_encoder
+from pyasn1.type import univ
 
 from nova import context
 from nova import db
 from nova import exception
-from nova import flags
-from nova.openstack.common import cfg
+from nova.openstack.common import excutils
+from nova.openstack.common import fileutils
 from nova.openstack.common import log as logging
 from nova.openstack.common import timeutils
+from nova import paths
 from nova import utils
 
 
@@ -51,10 +59,10 @@ crypto_opts = [
                default='crl.pem',
                help=_('Filename of root Certificate Revocation List')),
     cfg.StrOpt('keys_path',
-               default='$state_path/keys',
+               default=paths.state_path_def('keys'),
                help=_('Where we keep our keys')),
     cfg.StrOpt('ca_path',
-               default='$state_path/CA',
+               default=paths.state_path_def('CA'),
                help=_('Where we keep our root CA')),
     cfg.BoolOpt('use_project_ca',
                 default=False,
@@ -71,34 +79,34 @@ crypto_opts = [
                       'project, timestamp')),
     ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(crypto_opts)
+CONF = cfg.CONF
+CONF.register_opts(crypto_opts)
 
 
 def ca_folder(project_id=None):
-    if FLAGS.use_project_ca and project_id:
-        return os.path.join(FLAGS.ca_path, 'projects', project_id)
-    return FLAGS.ca_path
+    if CONF.use_project_ca and project_id:
+        return os.path.join(CONF.ca_path, 'projects', project_id)
+    return CONF.ca_path
 
 
 def ca_path(project_id=None):
-    return os.path.join(ca_folder(project_id), FLAGS.ca_file)
+    return os.path.join(ca_folder(project_id), CONF.ca_file)
 
 
 def key_path(project_id=None):
-    return os.path.join(ca_folder(project_id), FLAGS.key_file)
+    return os.path.join(ca_folder(project_id), CONF.key_file)
 
 
 def crl_path(project_id=None):
-    return os.path.join(ca_folder(project_id), FLAGS.crl_file)
+    return os.path.join(ca_folder(project_id), CONF.crl_file)
 
 
 def fetch_ca(project_id=None):
-    if not FLAGS.use_project_ca:
+    if not CONF.use_project_ca:
         project_id = None
     ca_file_path = ca_path(project_id)
     if not os.path.exists(ca_file_path):
-        raise exception.CryptoCAFileNotFound(project_id=project_id)
+        raise exception.CryptoCAFileNotFound(project=project_id)
     with open(ca_file_path, 'r') as cafile:
         return cafile.read()
 
@@ -107,12 +115,11 @@ def ensure_ca_filesystem():
     """Ensure the CA filesystem exists."""
     ca_dir = ca_folder()
     if not os.path.exists(ca_path()):
-        genrootca_sh_path = os.path.join(os.path.dirname(__file__),
-                                         'CA',
-                                         'genrootca.sh')
+        genrootca_sh_path = os.path.abspath(
+                os.path.join(os.path.dirname(__file__), 'CA', 'genrootca.sh'))
 
         start = os.getcwd()
-        utils.ensure_tree(ca_dir)
+        fileutils.ensure_tree(ca_dir)
         os.chdir(ca_dir)
         utils.execute("sh", genrootca_sh_path)
         os.chdir(start)
@@ -135,13 +142,14 @@ def generate_fingerprint(public_key):
             raise exception.InvalidKeypair()
 
 
-def generate_key_pair(bits=1024):
-    # what is the magic 65537?
-
+def generate_key_pair(bits=None):
     with utils.tempdir() as tmpdir:
         keyfile = os.path.join(tmpdir, 'temp')
-        utils.execute('ssh-keygen', '-q', '-b', bits, '-N', '',
-                      '-t', 'rsa', '-f', keyfile, '-C', 'Generated by Nova')
+        args = ['ssh-keygen', '-q', '-N', '', '-t', 'rsa',
+                '-f', keyfile, '-C', 'Generated by Nova']
+        if bits is not None:
+            args.extend(['-b', bits])
+        utils.execute(*args)
         fingerprint = _generate_fingerprint('%s.pub' % (keyfile))
         if not os.path.exists(keyfile):
             raise exception.FileNotFound(keyfile)
@@ -156,11 +164,11 @@ def generate_key_pair(bits=1024):
 
 def fetch_crl(project_id):
     """Get crl file for project."""
-    if not FLAGS.use_project_ca:
+    if not CONF.use_project_ca:
         project_id = None
     crl_file_path = crl_path(project_id)
     if not os.path.exists(crl_file_path):
-        raise exception.CryptoCRLFileNotFound(project_id)
+        raise exception.CryptoCRLFileNotFound(project=project_id)
     with open(crl_file_path, 'r') as crlfile:
         return crlfile.read()
 
@@ -171,13 +179,96 @@ def decrypt_text(project_id, text):
         raise exception.ProjectNotFound(project_id=project_id)
     try:
         dec, _err = utils.execute('openssl',
-                                 'rsautl',
-                                 '-decrypt',
-                                 '-inkey', '%s' % private_key,
-                                 process_input=text)
+                                  'rsautl',
+                                  '-decrypt',
+                                  '-inkey', '%s' % private_key,
+                                  process_input=text)
         return dec
-    except exception.ProcessExecutionError:
-        raise exception.DecryptionFailure()
+    except exception.ProcessExecutionError as exc:
+        raise exception.DecryptionFailure(reason=exc.stderr)
+
+
+_RSA_OID = univ.ObjectIdentifier('1.2.840.113549.1.1.1')
+
+
+def _to_sequence(*vals):
+    seq = univ.Sequence()
+    for i in range(len(vals)):
+        seq.setComponentByPosition(i, vals[i])
+    return seq
+
+
+def convert_from_sshrsa_to_pkcs8(pubkey):
+    """Convert a ssh public key to openssl format
+       Equivalent to the ssh-keygen's -m option
+    """
+    # get the second field from the public key file.
+    try:
+        keydata = base64.b64decode(pubkey.split(None)[1])
+    except IndexError:
+        msg = _("Unable to find the key")
+        raise exception.EncryptionFailure(reason=msg)
+
+    # decode the parts of the key
+    parts = []
+    while keydata:
+        dlen = struct.unpack('>I', keydata[:4])[0]
+        data = keydata[4:dlen + 4]
+        keydata = keydata[4 + dlen:]
+        parts.append(data)
+
+    # Use asn to build the openssl key structure
+    #
+    #  SEQUENCE(2 elem)
+    #    +- SEQUENCE(2 elem)
+    #    |    +- OBJECT IDENTIFIER (1.2.840.113549.1.1.1)
+    #    |    +- NULL
+    #    +- BIT STRING(1 elem)
+    #         +- SEQUENCE(2 elem)
+    #              +- INTEGER(2048 bit)
+    #              +- INTEGER 65537
+
+    # Build the sequence for the bit string
+    n_val = eval(
+        '0x' + ''.join(['%02X' % struct.unpack('B', x)[0] for x in parts[2]]))
+    e_val = eval(
+        '0x' + ''.join(['%02X' % struct.unpack('B', x)[0] for x in parts[1]]))
+    pkinfo = _to_sequence(univ.Integer(n_val), univ.Integer(e_val))
+
+    # Convert the sequence into a bit string
+    pklong = long(der_encoder.encode(pkinfo).encode('hex'), 16)
+    pkbitstring = univ.BitString("'00%s'B" % bin(pklong)[2:])
+
+    # Build the key data structure
+    oid = _to_sequence(_RSA_OID, univ.Null())
+    pkcs1_seq = _to_sequence(oid, pkbitstring)
+    pkcs8 = base64.encodestring(der_encoder.encode(pkcs1_seq))
+
+    # Remove the embedded new line and format the key, each line
+    # should be 64 characters long
+    return ('-----BEGIN PUBLIC KEY-----\n%s\n-----END PUBLIC KEY-----\n' %
+            re.sub("(.{64})", "\\1\n", pkcs8.replace('\n', ''), re.DOTALL))
+
+
+def ssh_encrypt_text(ssh_public_key, text):
+    """Encrypt text with an ssh public key.
+    """
+    with utils.tempdir() as tmpdir:
+        sslkey = os.path.abspath(os.path.join(tmpdir, 'ssl.key'))
+        try:
+            out = convert_from_sshrsa_to_pkcs8(ssh_public_key)
+            with open(sslkey, 'w') as f:
+                f.write(out)
+            enc, _err = utils.execute('openssl',
+                                      'rsautl',
+                                      '-encrypt',
+                                      '-pubin',
+                                      '-inkey', sslkey,
+                                      '-keyform', 'PEM',
+                                      process_input=text)
+            return enc
+        except exception.ProcessExecutionError as exc:
+            raise exception.EncryptionFailure(reason=exc.stderr)
 
 
 def revoke_cert(project_id, file_name):
@@ -188,7 +279,7 @@ def revoke_cert(project_id, file_name):
     utils.execute('openssl', 'ca', '-config', './openssl.cnf', '-revoke',
                   file_name)
     utils.execute('openssl', 'ca', '-gencrl', '-config', './openssl.cnf',
-                  '-out', FLAGS.crl_file)
+                  '-out', CONF.crl_file)
     os.chdir(start)
 
 
@@ -218,12 +309,12 @@ def revoke_certs_by_user_and_project(user_id, project_id):
 
 def _project_cert_subject(project_id):
     """Helper to generate user cert subject."""
-    return FLAGS.project_cert_subject % (project_id, timeutils.isotime())
+    return CONF.project_cert_subject % (project_id, timeutils.isotime())
 
 
 def _user_cert_subject(user_id, project_id):
     """Helper to generate user cert subject."""
-    return FLAGS.user_cert_subject % (project_id, user_id, timeutils.isotime())
+    return CONF.user_cert_subject % (project_id, user_id, timeutils.isotime())
 
 
 def generate_x509_cert(user_id, project_id, bits=1024):
@@ -250,9 +341,8 @@ def generate_x509_cert(user_id, project_id, bits=1024):
 
 def _ensure_project_folder(project_id):
     if not os.path.exists(ca_path(project_id)):
-        geninter_sh_path = os.path.join(os.path.dirname(__file__),
-                                        'CA',
-                                        'geninter.sh')
+        geninter_sh_path = os.path.abspath(
+                os.path.join(os.path.dirname(__file__), 'CA', 'geninter.sh'))
         start = os.getcwd()
         os.chdir(ca_folder())
         utils.execute('sh', geninter_sh_path, project_id,
@@ -280,7 +370,7 @@ def generate_vpn_files(project_id):
 
 
 def sign_csr(csr_text, project_id=None):
-    if not FLAGS.use_project_ca:
+    if not CONF.use_project_ca:
         project_id = None
     if not project_id:
         return _sign_csr(csr_text, ca_folder())
@@ -294,14 +384,18 @@ def _sign_csr(csr_text, ca_folder):
         inbound = os.path.join(tmpdir, 'inbound.csr')
         outbound = os.path.join(tmpdir, 'outbound.csr')
 
-        with open(inbound, 'w') as csrfile:
-            csrfile.write(csr_text)
+        try:
+            with open(inbound, 'w') as csrfile:
+                csrfile.write(csr_text)
+        except IOError:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_('Failed to write inbound.csr'))
 
         LOG.debug(_('Flags path: %s'), ca_folder)
         start = os.getcwd()
 
         # Change working dir to CA
-        utils.ensure_tree(ca_folder)
+        fileutils.ensure_tree(ca_folder)
         os.chdir(ca_folder)
         utils.execute('openssl', 'ca', '-batch', '-out', outbound, '-config',
                       './openssl.cnf', '-infiles', inbound)
