@@ -12,25 +12,59 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-try:
-    import ldap
-except ImportError:
-    # This module needs to be importable despite ldap not being a requirement
-    ldap = None
-
+import ldap
 import time
 
-from oslo_log import log as logging
+from oslo.config import cfg
 
-import nova.conf
 from nova import exception
-from nova.i18n import _
 from nova.network import dns_driver
+from nova.openstack.common import log as logging
 from nova import utils
 
-CONF = nova.conf.CONF
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
+ldap_dns_opts = [
+    cfg.StrOpt('ldap_dns_url',
+               default='ldap://ldap.example.com:389',
+               help='URL for ldap server which will store dns entries'),
+    cfg.StrOpt('ldap_dns_user',
+               default='uid=admin,ou=people,dc=example,dc=org',
+               help='user for ldap DNS'),
+    cfg.StrOpt('ldap_dns_password',
+               default='password',
+               help='password for ldap DNS',
+               secret=True),
+    cfg.StrOpt('ldap_dns_soa_hostmaster',
+               default='hostmaster@example.org',
+               help='Hostmaster for ldap dns driver Statement of Authority'),
+    cfg.MultiStrOpt('ldap_dns_servers',
+                    default=['dns.example.org'],
+                    help='DNS Servers for ldap dns driver'),
+    cfg.StrOpt('ldap_dns_base_dn',
+               default='ou=hosts,dc=example,dc=org',
+               help='Base DN for DNS entries in ldap'),
+    cfg.StrOpt('ldap_dns_soa_refresh',
+               default='1800',
+               help='Refresh interval (in seconds) for ldap dns driver '
+                    'Statement of Authority'),
+    cfg.StrOpt('ldap_dns_soa_retry',
+               default='3600',
+               help='Retry interval (in seconds) for ldap dns driver '
+                    'Statement of Authority'),
+    cfg.StrOpt('ldap_dns_soa_expiry',
+               default='86400',
+               help='Expiry interval (in seconds) for ldap dns driver '
+                    'Statement of Authority'),
+    cfg.StrOpt('ldap_dns_soa_minimum',
+               default='7200',
+               help='Minimum interval (in seconds) for ldap dns driver '
+                    'Statement of Authority'),
+    ]
+
+CONF.register_opts(ldap_dns_opts)
+
 
 # Importing ldap.modlist breaks the tests for some reason,
 #  so this is an abbreviated version of a function from
@@ -50,10 +84,8 @@ class DNSEntry(object):
 
     def __init__(self, ldap_object):
         """ldap_object is an instance of ldap.LDAPObject.
-
            It should already be initialized and bound before
-           getting passed in here.
-        """
+           getting passed in here."""
         self.lobj = ldap_object
         self.ldap_tuple = None
         self.qualified_domain = None
@@ -65,9 +97,9 @@ class DNSEntry(object):
         if not entry:
             return None
         if len(entry) > 1:
-            LOG.warning("Found multiple matches for domain "
-                        "%(domain)s.\n%(entry)s",
-                        domain, entry)
+            LOG.warn(_("Found multiple matches for domain "
+                    "%(domain)s.\n%(entry)s") %
+                    (domain, entry))
         return entry[0]
 
     @classmethod
@@ -92,10 +124,10 @@ class DNSEntry(object):
         if name.endswith(z):
             dequalified = name[0:name.rfind(z)]
         else:
-            LOG.warning("Unable to dequalify.  %(name)s is not in "
-                        "%(domain)s.\n",
-                        {'name': name,
-                         'domain': self.qualified_domain})
+            LOG.warn(_("Unable to dequalify.  %(name)s is not in "
+                       "%(domain)s.\n") %
+                     {'name': name,
+                      'domain': self.qualified_domain})
             dequalified = None
 
         return dequalified
@@ -114,7 +146,7 @@ class DomainEntry(DNSEntry):
     @classmethod
     def _soa(cls):
         date = time.strftime('%Y%m%d%H%M%S')
-        soa = '%s %s %s %d %d %d %d' % (
+        soa = '%s %s %s %s %s %s %s' % (
                  CONF.ldap_dns_servers[0],
                  CONF.ldap_dns_soa_hostmaster,
                  date,
@@ -208,6 +240,7 @@ class DomainEntry(DNSEntry):
                      'dc': [name]}
             self.lobj.add_s(newdn, create_modlist(attrs))
             return self.subentry_with_name(name)
+        self.update_soa()
 
     def remove_entry(self, name):
         entry = self.subentry_with_name(name)
@@ -274,13 +307,9 @@ class LdapDNS(dns_driver.DNSDriver):
     """Driver for PowerDNS using ldap as a back end.
 
        This driver assumes ldap-method=strict, with all domains
-       in the top-level, aRecords only.
-    """
+       in the top-level, aRecords only."""
 
     def __init__(self):
-        if not ldap:
-            raise ImportError(_('ldap not installed'))
-
         self.lobj = ldap.initialize(CONF.ldap_dns_url)
         self.lobj.simple_bind_s(CONF.ldap_dns_user,
                                 CONF.ldap_dns_password)
@@ -333,5 +362,5 @@ class LdapDNS(dns_driver.DNSDriver):
         dEntry.delete()
 
     def delete_dns_file(self):
-        LOG.warning("This shouldn't be getting called except during testing.")
+        LOG.warn(_("This shouldn't be getting called except during testing."))
         pass
