@@ -373,12 +373,28 @@ class NetworkManager(manager.Manager):
         # thus enabling `read_deleted`
         admin_context = context.get_admin_context(read_deleted='yes')
         if uuidutils.is_uuid_like(instance_id):
-            instance_ref = self.db.instance_get_by_uuid(admin_context,
-                                                        instance_id)
+            instance = self.db.instance_get_by_uuid(admin_context,
+                                                    instance_id)
         else:
-            instance_ref = self.db.instance_get(admin_context, instance_id)
+            instance = self.db.instance_get(admin_context,
+                                            instance_id)
 
-        groups = instance_ref['security_groups']
+        try:
+            # NOTE(vish): We need to make sure the instance info cache has been
+            #             updated with new ip info before we trigger the
+            #             security group refresh. This is somewhat ineffecient
+            #             but avoids doing some dangerous refactoring for a
+            #             bug fix.
+            nw_info = self.get_instance_nw_info(admin_context,
+                                                instance['uuid'],
+                                                None, None)
+            cache = {'network_info': nw_info.json()}
+            self.db.instance_info_cache_update(admin_context,
+                                               instance['uuid'],
+                                               cache)
+        except exception.InstanceInfoCacheNotFound:
+            pass
+        groups = instance['security_groups']
         group_ids = [group['id'] for group in groups]
 
         self.security_group_api.trigger_members_refresh(admin_context,
@@ -846,15 +862,15 @@ class NetworkManager(manager.Manager):
                 else:
                     address = self.db.fixed_ip_associate_pool(
                         context.elevated(), network['id'], instance_id)
-                self._do_trigger_security_group_members_refresh_for_instance(
-                    instance_id)
-                self._do_trigger_security_group_handler(
-                    'instance_add_security_group', instance_id)
                 get_vif = self.db.virtual_interface_get_by_instance_and_network
                 vif = get_vif(context, instance_id, network['id'])
                 values = {'allocated': True,
                           'virtual_interface_id': vif['id']}
                 self.db.fixed_ip_update(context, address, values)
+                self._do_trigger_security_group_members_refresh_for_instance(
+                    instance_id)
+                self._do_trigger_security_group_handler(
+                    'instance_add_security_group', instance_id)
 
             # NOTE(vish) This db query could be removed if we pass az and name
             #            (or the whole instance object).
@@ -1717,8 +1733,6 @@ class VlanManager(RPCAllocateFixedIP, floating_ips.FloatingIP, NetworkManager):
                 address = self.db.fixed_ip_associate_pool(context,
                                                           network['id'],
                                                           instance_id)
-            self._do_trigger_security_group_members_refresh_for_instance(
-                                                                   instance_id)
 
         vif = self.db.virtual_interface_get_by_instance_and_network(
             context, instance_id, network['id'])
@@ -1726,6 +1740,10 @@ class VlanManager(RPCAllocateFixedIP, floating_ips.FloatingIP, NetworkManager):
                   'virtual_interface_id': vif['id']}
         self.db.fixed_ip_update(context, address, values)
 
+        if not kwargs.get('vpn', None):
+            self._do_trigger_security_group_members_refresh_for_instance(
+                                                                   instance_id)
+
         # NOTE(vish) This db query could be removed if we pass az and name
         #            (or the whole instance object).
         instance = self.db.instance_get_by_uuid(context, instance_id)
