@@ -1,4 +1,4 @@
-# Copyright (c) 2011-2012 OpenStack, LLC.
+# Copyright (c) 2011-2012 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -13,12 +13,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.config import cfg
 
+from nova import db
 from nova.scheduler import filters
 
+CONF = cfg.CONF
+CONF.import_opt('default_availability_zone', 'nova.availability_zones')
+
 
 class AvailabilityZoneFilter(filters.BaseHostFilter):
-    """Filters Hosts by availability zone."""
+    """Filters Hosts by availability zone.
+
+    Works with aggregate metadata availability zones, using the key
+    'availability_zone'
+    Note: in theory a compute node can be part of multiple availability_zones
+    """
 
     def host_passes(self, host_state, filter_properties):
         spec = filter_properties.get('request_spec', {})
@@ -26,5 +36,12 @@ class AvailabilityZoneFilter(filters.BaseHostFilter):
         availability_zone = props.get('availability_zone')
 
         if availability_zone:
-            return availability_zone == host_state.service['availability_zone']
+            context = filter_properties['context'].elevated()
+            metadata = db.aggregate_metadata_get_by_host(
+                         context, host_state.host, key='availability_zone')
+            if 'availability_zone' in metadata:
+                return availability_zone in metadata['availability_zone']
+            else:
+                return availability_zone == CONF.default_availability_zone
+
         return True
