@@ -1,6 +1,6 @@
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010 OpenStack Foundation
 # Copyright 2010 United States Government as represented by the
 # Administrator of the National Aeronautics and Space Administration.
 # All Rights Reserved.
@@ -19,10 +19,16 @@
 
 """
 Scheduler that allows routing some calls to one driver and others to another.
+
+This scheduler was originally used to deal with both compute and volume. But
+is now used for openstack extensions that want to use the nova-scheduler to
+schedule requests to compute nodes but provide their own manager and topic.
+
+https://bugs.launchpad.net/nova/+bug/1009681
 """
 
-from nova import flags
-from nova.openstack.common import cfg
+from oslo.config import cfg
+
 from nova.openstack.common import importutils
 from nova.scheduler import driver
 
@@ -32,16 +38,13 @@ multi_scheduler_opts = [
                default='nova.scheduler.'
                     'filter_scheduler.FilterScheduler',
                help='Driver to use for scheduling compute calls'),
-    cfg.StrOpt('volume_scheduler_driver',
-               default='nova.scheduler.chance.ChanceScheduler',
-               help='Driver to use for scheduling volume calls'),
     cfg.StrOpt('default_scheduler_driver',
                default='nova.scheduler.chance.ChanceScheduler',
                help='Default driver to use for scheduling calls'),
     ]
 
-FLAGS = flags.FLAGS
-FLAGS.register_opts(multi_scheduler_opts)
+CONF = cfg.CONF
+CONF.register_opts(multi_scheduler_opts)
 
 
 class MultiScheduler(driver.Scheduler):
@@ -55,14 +58,11 @@ class MultiScheduler(driver.Scheduler):
     def __init__(self):
         super(MultiScheduler, self).__init__()
         compute_driver = importutils.import_object(
-                FLAGS.compute_scheduler_driver)
-        volume_driver = importutils.import_object(
-                FLAGS.volume_scheduler_driver)
+                CONF.compute_scheduler_driver)
         default_driver = importutils.import_object(
-                FLAGS.default_scheduler_driver)
+                CONF.default_scheduler_driver)
 
         self.drivers = {'compute': compute_driver,
-                        'volume': volume_driver,
                         'default': default_driver}
 
     def schedule_run_instance(self, *args, **kwargs):
@@ -71,9 +71,6 @@ class MultiScheduler(driver.Scheduler):
     def schedule_prep_resize(self, *args, **kwargs):
         return self.drivers['compute'].schedule_prep_resize(*args, **kwargs)
 
-    def schedule_create_volume(self, *args, **kwargs):
-        return self.drivers['volume'].schedule_create_volume(*args, **kwargs)
-
     def update_service_capabilities(self, service_name, host, capabilities):
         # Multi scheduler is only a holder of sub-schedulers, so
         # pass the capabilities to the schedulers that matter
