@@ -17,97 +17,92 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import iso8601
-from oslo_log import log as logging
-from oslo_utils import timeutils
+from oslo.config import cfg
 
-from nova import cache_utils
-import nova.conf
-from nova.i18n import _, _LI, _LW
+from nova import conductor
+from nova import context
+from nova.openstack.common import log as logging
+from nova.openstack.common import memorycache
+from nova.openstack.common import timeutils
 from nova.servicegroup import api
-from nova.servicegroup.drivers import base
+from nova import utils
 
 
-CONF = nova.conf.CONF
+CONF = cfg.CONF
+CONF.import_opt('service_down_time', 'nova.service')
+CONF.import_opt('memcached_servers', 'nova.openstack.common.memorycache')
 
 
 LOG = logging.getLogger(__name__)
 
 
-class MemcachedDriver(base.Driver):
+class MemcachedDriver(api.ServiceGroupDriver):
 
     def __init__(self, *args, **kwargs):
-        self.mc = cache_utils.get_memcached_client(
-                expiration_time=CONF.service_down_time)
+        test = kwargs.get('test')
+        if not CONF.memcached_servers and not test:
+            raise RuntimeError(_('memcached_servers not defined'))
+        self.mc = memorycache.get_client()
+        self.db_allowed = kwargs.get('db_allowed', True)
+        self.conductor_api = conductor.API(use_local=self.db_allowed)
 
     def join(self, member_id, group_id, service=None):
         """Join the given service with its group."""
 
-        LOG.debug('Memcached_Driver: join new ServiceGroup member '
-                  '%(member_id)s to the %(group_id)s group, '
-                  'service = %(service)s',
-                  {'member_id': member_id,
-                   'group_id': group_id,
-                   'service': service})
+        msg = _('Memcached_Driver: join new ServiceGroup member '
+                '%(member_id)s to the %(group_id)s group, '
+                'service = %(service)s')
+        LOG.debug(msg, locals())
         if service is None:
             raise RuntimeError(_('service is a mandatory argument for '
                                  'Memcached based ServiceGroup driver'))
         report_interval = service.report_interval
         if report_interval:
-            service.tg.add_timer_args(
-                report_interval, self._report_state, args=[service],
-                initial_delay=api.INITIAL_REPORTING_DELAY)
+            pulse = utils.FixedIntervalLoopingCall(self._report_state, service)
+            pulse.start(interval=report_interval,
+                        initial_delay=report_interval)
+            return pulse
 
     def is_up(self, service_ref):
         """Moved from nova.utils
         Check whether a service is up based on last heartbeat.
         """
         key = "%(topic)s:%(host)s" % service_ref
-        is_up = self.mc.get(str(key)) is not None
-        if not is_up:
-            LOG.debug('Seems service %s is down', key)
+        return self.mc.get(str(key)) is not None
 
-        return is_up
-
-    def updated_time(self, service_ref):
-        """Get the updated time from memcache"""
-        key = "%(topic)s:%(host)s" % service_ref
-        updated_time_in_mc = self.mc.get(str(key))
-        updated_time_in_db = service_ref['updated_at']
-
-        if updated_time_in_mc:
-            # Change mc time to offset-aware time
-            updated_time_in_mc = \
-                updated_time_in_mc.replace(tzinfo=iso8601.UTC)
-            # If [DEFAULT]/enable_new_services is set to be false, the
-            # ``updated_time_in_db`` will be None, in this case, use
-            # ``updated_time_in_mc`` instead.
-            if (not updated_time_in_db or
-                    updated_time_in_db <= updated_time_in_mc):
-                return updated_time_in_mc
-
-        return updated_time_in_db
+    def get_all(self, group_id):
+        """
+        Returns ALL members of the given group
+        """
+        LOG.debug(_('Memcached_Driver: get_all members of the %s group') %
+                  group_id)
+        rs = []
+        ctxt = context.get_admin_context()
+        services = self.conductor_api.service_get_all_by_topic(ctxt, group_id)
+        for service in services:
+            if self.is_up(service):
+                rs.append(service['host'])
+        return rs
 
     def _report_state(self, service):
         """Update the state of this service in the datastore."""
+        ctxt = context.get_admin_context()
         try:
             key = "%(topic)s:%(host)s" % service.service_ref
             # memcached has data expiration time capability.
             # set(..., time=CONF.service_down_time) uses it and
             # reduces key-deleting code.
             self.mc.set(str(key),
-                        timeutils.utcnow())
+                        timeutils.utcnow(),
+                        time=CONF.service_down_time)
 
             # TODO(termie): make this pattern be more elegant.
             if getattr(service, 'model_disconnected', False):
                 service.model_disconnected = False
-                LOG.info(
-                    _LI('Recovered connection to memcache server '
-                        'for reporting service status.'))
+                LOG.error(_('Recovered model server connection!'))
 
         # TODO(vish): this should probably only catch connection errors
-        except Exception:
+        except Exception:  # pylint: disable=W0702
             if not getattr(service, 'model_disconnected', False):
                 service.model_disconnected = True
-                LOG.warning(_LW('Lost connection to memcache server '
-                             'for reporting service status.'))
+                LOG.exception(_('model server went away'))
