@@ -16,7 +16,7 @@
 #    under the License.
 
 
-from nova.openstack.common.log import logging
+from nova.openstack.common import log as logging
 from nova.tests.integrated import integrated_helpers
 
 
@@ -25,7 +25,7 @@ LOG = logging.getLogger(__name__)
 
 class LoginTest(integrated_helpers._IntegratedTestBase):
     def test_login(self):
-        """Simple check - we list flavors - so we know we're logged in."""
+        # Simple check - we list flavors - so we know we're logged in.
         flavors = self.api.get_flavors()
         for flavor in flavors:
             LOG.debug(_("flavor: %s") % flavor)
