@@ -1045,6 +1045,18 @@ class TestQuantumv2(test.TestCase):
                           api.get_floating_ip_by_address,
                           self.context, address)
 
+    def test_get_floating_ip_by_id_not_found(self):
+        api = quantumapi.API()
+        QuantumNotFound = qexceptions.QuantumClientException(
+            status_code=404)
+        floating_ip_id = self.fip_unassociated['id']
+        self.moxed_client.show_floatingip(floating_ip_id).\
+            AndRaise(QuantumNotFound)
+        self.mox.ReplayAll()
+        self.assertRaises(exception.FloatingIpNotFound,
+                          api.get_floating_ip,
+                          self.context, floating_ip_id)
+
     def test_get_floating_ip_by_address_multiple_found(self):
         api = quantumapi.API()
         address = self.fip_unassociated['floating_ip_address']
