@@ -16,19 +16,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Unit tests for the API endpoint"""
+"""Unit tests for the API endpoint."""
 
+import pkg_resources
 import random
 import StringIO
 
 import boto
+import boto.connection
 from boto.ec2 import regioninfo
 from boto import exception as boto_exc
 # newer versions of boto use their own wrapper on top of httplib.HTTPResponse
-try:
-    from boto.connection import HTTPResponse
-except ImportError:
-    from httplib import HTTPResponse
+if hasattr(boto.connection, 'HTTPResponse'):
+    httplib = boto.connection
+else:
+    import httplib
+import fixtures
 import webob
 
 from nova.api import auth
@@ -38,22 +41,19 @@ from nova.api.ec2 import ec2utils
 from nova import block_device
 from nova import context
 from nova import exception
-from nova import flags
 from nova.openstack.common import timeutils
 from nova import test
-
-
-FLAGS = flags.FLAGS
+from nova.tests import matchers
 
 
 class FakeHttplibSocket(object):
-    """a fake socket implementation for httplib.HTTPResponse, trivial"""
+    """a fake socket implementation for httplib.HTTPResponse, trivial."""
     def __init__(self, response_string):
         self.response_string = response_string
         self._buffer = StringIO.StringIO(response_string)
 
     def makefile(self, _mode, _other):
-        """Returns the socket's internal buffer"""
+        """Returns the socket's internal buffer."""
         return self._buffer
 
 
@@ -81,7 +81,7 @@ class FakeHttplibConnection(object):
         # guess that's a function the web server usually provides.
         resp = "HTTP/1.0 %s" % resp
         self.sock = FakeHttplibSocket(resp)
-        self.http_response = HTTPResponse(self.sock)
+        self.http_response = httplib.HTTPResponse(self.sock)
         # NOTE(vish): boto is accessing private variables for some reason
         self._HTTPConnection__response = self.http_response
         self.http_response.begin()
@@ -93,12 +93,12 @@ class FakeHttplibConnection(object):
         return self.sock.response_string
 
     def close(self):
-        """Required for compatibility with boto/tornado"""
+        """Required for compatibility with boto/tornado."""
         pass
 
 
 class XmlConversionTestCase(test.TestCase):
-    """Unit test api xml conversion"""
+    """Unit test api xml conversion."""
     def test_number_conversion(self):
         conv = ec2utils._try_convert
         self.assertEqual(conv('None'), None)
@@ -163,7 +163,7 @@ class Ec2utilsTestCase(test.TestCase):
                   'virtual_name': 'ephemeral0'}}}
         out_dict = ec2utils.dict_from_dotted_str(in_str)
 
-        self.assertDictMatch(out_dict, expected_dict)
+        self.assertThat(out_dict, matchers.DictMatches(expected_dict))
 
     def test_properties_root_defice_name(self):
         mappings = [{"device": "/dev/sda1", "virtual": "root"}]
@@ -209,12 +209,12 @@ class Ec2utilsTestCase(test.TestCase):
              'device': '/dev/sdc1'},
             {'virtual': 'ephemeral1',
              'device': '/dev/sdc1'}]
-        self.assertDictListMatch(block_device.mappings_prepend_dev(mappings),
-                                 expected_result)
+        self.assertThat(block_device.mappings_prepend_dev(mappings),
+                        matchers.DictListMatches(expected_result))
 
 
 class ApiEc2TestCase(test.TestCase):
-    """Unit test for the cloud controller on an EC2 API"""
+    """Unit test for the cloud controller on an EC2 API."""
     def setUp(self):
         super(ApiEc2TestCase, self).setUp()
         self.host = '127.0.0.1'
@@ -224,9 +224,10 @@ class ApiEc2TestCase(test.TestCase):
         self.app = auth.InjectContext(ctxt, ec2.FaultWrapper(
                 ec2.RequestLogging(ec2.Requestify(ec2.Authorizer(ec2.Executor()
                                ), 'nova.api.ec2.cloud.CloudController'))))
+        self.useFixture(fixtures.FakeLogger('boto'))
 
     def expect_http(self, host=None, is_secure=False, api_version=None):
-        """Returns a new EC2 connection"""
+        """Returns a new EC2 connection."""
         self.ec2 = boto.connect_ec2(
                 aws_access_key_id='fake',
                 aws_secret_access_key='fake',
@@ -241,7 +242,11 @@ class ApiEc2TestCase(test.TestCase):
         self.http = FakeHttplibConnection(
                 self.app, '%s:8773' % (self.host), False)
         # pylint: disable=E1103
-        if boto.Version >= '2':
+        boto_version = pkg_resources.parse_version(boto.Version)
+        if boto_version >= pkg_resources.parse_version('2.13'):
+            self.ec2.new_http_connection(host or self.host, 8773,
+                is_secure).AndReturn(self.http)
+        elif boto_version >= pkg_resources.parse_version('2'):
             self.ec2.new_http_connection(host or '%s:8773' % (self.host),
                 is_secure).AndReturn(self.http)
         else:
@@ -282,7 +287,7 @@ class ApiEc2TestCase(test.TestCase):
         self.assertEqual(self.ec2.get_all_instances(), [])
 
     def test_terminate_invalid_instance(self):
-        """Attempt to terminate an invalid instance"""
+        # Attempt to terminate an invalid instance.
         self.expect_http()
         self.mox.ReplayAll()
         self.assertRaises(boto_exc.EC2ResponseError,
@@ -310,16 +315,15 @@ class ApiEc2TestCase(test.TestCase):
         try:
             self.ec2.create_key_pair('test')
         except boto_exc.EC2ResponseError, e:
-            if e.code == 'KeyPairExists':
+            if e.code == 'InvalidKeyPair.Duplicate':
                 pass
             else:
-                self.fail("Unexpected EC2ResponseError: %s "
-                          "(expected KeyPairExists)" % e.code)
+                self.assertEqual('InvalidKeyPair.Duplicate', e.code)
         else:
             self.fail('Exception not raised.')
 
     def test_get_all_security_groups(self):
-        """Test that we can retrieve security groups"""
+        # Test that we can retrieve security groups.
         self.expect_http()
         self.mox.ReplayAll()
 
@@ -329,7 +333,7 @@ class ApiEc2TestCase(test.TestCase):
         self.assertEquals(rv[0].name, 'default')
 
     def test_create_delete_security_group(self):
-        """Test that we can create a security group"""
+        # Test that we can create a security group.
         self.expect_http()
         self.mox.ReplayAll()
 
@@ -351,7 +355,7 @@ class ApiEc2TestCase(test.TestCase):
         self.ec2.delete_security_group(security_group_name)
 
     def test_group_name_valid_chars_security_group(self):
-        """ Test that we sanely handle invalid security group names.
+        """Test that we sanely handle invalid security group names.
          EC2 API Spec states we should only accept alphanumeric characters,
          spaces, dashes, and underscores. Amazon implementation
          accepts more characters - so, [:print:] is ok. """
@@ -366,7 +370,7 @@ class ApiEc2TestCase(test.TestCase):
         for test in test_raise:
             self.expect_http()
             self.mox.ReplayAll()
-            FLAGS.ec2_strict_validation = test[0]
+            self.flags(ec2_strict_validation=test[0])
             self.assertRaises(boto_exc.EC2ResponseError,
                               self.ec2.create_security_group,
                               test[1],
@@ -378,7 +382,7 @@ class ApiEc2TestCase(test.TestCase):
         for test in test_accept:
             self.expect_http()
             self.mox.ReplayAll()
-            FLAGS.ec2_strict_validation = test[0]
+            self.flags(ec2_strict_validation=test[0])
             self.ec2.create_security_group(test[1], test[2])
             self.expect_http()
             self.mox.ReplayAll()
@@ -431,7 +435,7 @@ class ApiEc2TestCase(test.TestCase):
                 group.authorize(*args)
             except boto_exc.EC2ResponseError as e:
                 self.assertEqual(e.status, 400, 'Expected status to be 400')
-                self.assertIn(message, e.error_message, e.error_message)
+                self.assertIn(message, e.error_message)
             else:
                 raise self.failureException, 'EC2ResponseError not raised'
 
