@@ -92,7 +92,7 @@ class LibvirtBaseVIFDriver(object):
             return mapping['vif_devname']
         return ("nic" + mapping['vif_uuid'])[:network_model.NIC_NAME_LEN]
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         conf = vconfig.LibvirtConfigGuestInterface()
         # Default to letting libvirt / the hypervisor choose the model
         model = None
@@ -159,13 +159,14 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
             return True
         return False
 
-    def get_config_bridge(self, instance, network, mapping, image_meta):
+    def get_config_bridge(self, instance, network, mapping, image_meta,
+                          inst_type):
         """Get VIF configurations for bridge type."""
         conf = super(LibvirtGenericVIFDriver,
                      self).get_config(instance,
                                       network,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
         designer.set_vif_host_backend_bridge_config(
             conf, self.get_bridge_name(network),
@@ -175,28 +176,30 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
         name = "nova-instance-" + instance['name'] + "-" + mac_id
         if self.get_firewall_required():
             conf.filtername = name
-        designer.set_vif_bandwidth_config(conf, instance)
+        designer.set_vif_bandwidth_config(conf, inst_type)
 
         return conf
 
-    def get_config_ovs_ethernet(self, instance, network, mapping, image_meta):
+    def get_config_ovs_ethernet(self, instance, network, mapping,
+                                image_meta, inst_type):
         conf = super(LibvirtGenericVIFDriver,
                      self).get_config(instance,
                                       network,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
         dev = self.get_vif_devname(mapping)
         designer.set_vif_host_backend_ethernet_config(conf, dev)
 
         return conf
 
-    def get_config_ovs_bridge(self, instance, network, mapping, image_meta):
+    def get_config_ovs_bridge(self, instance, network, mapping, image_meta,
+                              inst_type):
         conf = super(LibvirtGenericVIFDriver,
                      self).get_config(instance,
                                       network,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
         designer.set_vif_host_backend_ovs_config(
             conf, self.get_bridge_name(network),
@@ -205,34 +208,40 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
 
         return conf
 
-    def get_config_ovs_hybrid(self, instance, network, mapping, image_meta):
+    def get_config_ovs_hybrid(self, instance, network, mapping, image_meta,
+                              inst_type):
         newnet = copy.deepcopy(network)
         newnet['bridge'] = self.get_br_name(mapping['vif_uuid'])
         return self.get_config_bridge(instance,
                                       newnet,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
-    def get_config_ovs(self, instance, network, mapping, image_meta):
+    def get_config_ovs(self, instance, network, mapping, image_meta,
+                       inst_type):
         if self.get_firewall_required():
             return self.get_config_ovs_hybrid(instance, network,
                                               mapping,
-                                              image_meta)
+                                              image_meta,
+                                              inst_type)
         elif self.has_libvirt_version(LIBVIRT_OVS_VPORT_VERSION):
             return self.get_config_ovs_bridge(instance, network,
                                               mapping,
-                                              image_meta)
+                                              image_meta,
+                                              inst_type)
         else:
             return self.get_config_ovs_ethernet(instance, network,
                                                 mapping,
-                                                image_meta)
+                                                image_meta,
+                                                inst_type)
 
-    def get_config_802qbg(self, instance, network, mapping, image_meta):
+    def get_config_802qbg(self, instance, network, mapping, image_meta,
+                          inst_type):
         conf = super(LibvirtGenericVIFDriver,
                      self).get_config(instance,
                                       network,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
         params = mapping["qbg_params"]
         designer.set_vif_host_backend_802qbg_config(
@@ -244,12 +253,13 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
 
         return conf
 
-    def get_config_802qbh(self, instance, network, mapping, image_meta):
+    def get_config_802qbh(self, instance, network, mapping, image_meta,
+                          inst_type):
         conf = super(LibvirtGenericVIFDriver,
                      self).get_config(instance,
                                       network,
                                       mapping,
-                                      image_meta)
+                                      image_meta, inst_type)
 
         params = mapping["qbh_params"]
         designer.set_vif_host_backend_802qbh_config(
@@ -258,7 +268,7 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
 
         return conf
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         vif_type = mapping.get('vif_type')
 
         LOG.debug(_("vif_type=%(vif_type)s instance=%(instance)s "
@@ -273,19 +283,23 @@ class LibvirtGenericVIFDriver(LibvirtBaseVIFDriver):
         if vif_type == network_model.VIF_TYPE_BRIDGE:
             return self.get_config_bridge(instance,
                                           network, mapping,
-                                          image_meta)
+                                          image_meta,
+                                          inst_type)
         elif vif_type == network_model.VIF_TYPE_OVS:
             return self.get_config_ovs(instance,
                                        network, mapping,
-                                       image_meta)
+                                       image_meta,
+                                       inst_type)
         elif vif_type == network_model.VIF_TYPE_802_QBG:
             return self.get_config_802qbg(instance,
                                           network, mapping,
-                                          image_meta)
+                                          image_meta,
+                                          inst_type)
         elif vif_type == network_model.VIF_TYPE_802_QBH:
             return self.get_config_802qbh(instance,
                                           network, mapping,
-                                          image_meta)
+                                          image_meta,
+                                          inst_type)
         else:
             raise exception.NovaException(
                 _("Unexpected vif_type=%s") % vif_type)
@@ -496,8 +510,9 @@ class LibvirtBridgeDriver(LibvirtGenericVIFDriver):
        drivers which do not yet report 'vif_type' port binding.
        Will be deprecated in Havana, and removed in Ixxxx."""
 
-    def get_config(self, instance, network, mapping, image_meta):
-        return self.get_config_bridge(instance, network, mapping, image_meta)
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
+        return self.get_config_bridge(instance, network, mapping, image_meta,
+                                      inst_type)
 
     def plug(self, instance, vif):
         self.plug_bridge(instance, vif)
@@ -517,10 +532,10 @@ class LibvirtOpenVswitchDriver(LibvirtGenericVIFDriver):
     def get_ovs_interfaceid(self, mapping):
         return mapping.get('ovs_interfaceid') or mapping['vif_uuid']
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         return self.get_config_ovs_ethernet(instance,
                                             network, mapping,
-                                            image_meta)
+                                            image_meta, inst_type)
 
     def plug(self, instance, vif):
         self.plug_ovs_ethernet(instance, vif)
@@ -540,10 +555,11 @@ class LibvirtHybridOVSBridgeDriver(LibvirtGenericVIFDriver):
     def get_ovs_interfaceid(self, mapping):
         return mapping.get('ovs_interfaceid') or mapping['vif_uuid']
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         return self.get_config_ovs_hybrid(instance,
                                           network, mapping,
-                                          image_meta)
+                                          image_meta,
+                                          inst_type)
 
     def plug(self, instance, vif):
         return self.plug_ovs_hybrid(instance, vif)
@@ -563,10 +579,11 @@ class LibvirtOpenVswitchVirtualPortDriver(LibvirtGenericVIFDriver):
     def get_ovs_interfaceid(self, mapping):
         return mapping.get('ovs_interfaceid') or mapping['vif_uuid']
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         return self.get_config_ovs_bridge(instance,
                                           network, mapping,
-                                          image_meta)
+                                          image_meta,
+                                          inst_type)
 
     def plug(self, instance, vif):
         return self.plug_ovs_bridge(instance, vif)
@@ -584,12 +601,13 @@ class QuantumLinuxBridgeVIFDriver(LibvirtGenericVIFDriver):
         def_bridge = ("brq" + network['id'])[:network_model.NIC_NAME_LEN]
         return network.get('bridge') or def_bridge
 
-    def get_config(self, instance, network, mapping, image_meta):
+    def get_config(self, instance, network, mapping, image_meta, inst_type):
         # In order for libvirt to make use of the bridge name then it has
         # to ensure that the bridge exists
         if 'should_create_bridge' not in mapping:
             mapping['should_create_bridge'] = True
-        return self.get_config_bridge(instance, network, mapping, image_meta)
+        return self.get_config_bridge(instance, network, mapping, image_meta,
+                                      inst_type)
 
     def plug(self, instance, vif):
         self.plug_bridge(instance, vif)
