@@ -163,44 +163,30 @@ class VCState(object):
         return self._stats
 
     def update_status(self):
-        """Update the current state of the host.
-        """
-        host_mor = vm_util.get_host_ref(self._session, self._cluster)
-        if host_mor is None:
-            return
-
-        summary = self._session._call_method(vim_util,
-                                             "get_dynamic_property",
-                                             host_mor,
-                                             "HostSystem",
-                                             "summary")
-
-        if summary is None:
-            return
-
+        """Update the current state of the cluster."""
+        # Get the datastore in the cluster
         try:
             ds = vm_util.get_datastore_ref_and_name(self._session,
                                                     self._cluster)
         except exception.DatastoreNotFound:
             ds = (None, None, 0, 0)
 
+        # Get cpu, memory stats from the cluster
+        stats = vm_util.get_stats_from_cluster(self._session, self._cluster)
+        about_info = self._session._call_method(vim_util, "get_about_info")
         data = {}
-        data["vcpus"] = summary.hardware.numCpuThreads
-        data["cpu_info"] =\
-        {"vendor": summary.hardware.vendor,
-         "model": summary.hardware.cpuModel,
-         "topology": {"cores": summary.hardware.numCpuCores,
-                      "sockets": summary.hardware.numCpuPkgs,
-                      "threads": summary.hardware.numCpuThreads}
-        }
+        data["vcpus"] = stats['cpu']['vcpus']
+        data["cpu_info"] = {"vendor": stats['cpu']['vendor'],
+                            "model": stats['cpu']['model'],
+                            "topology": {"cores": stats['cpu']['cores'],
+                                         "threads": stats['cpu']['vcpus']}}
         data["disk_total"] = ds[2] / (1024 * 1024 * 1024)
         data["disk_available"] = ds[3] / (1024 * 1024 * 1024)
         data["disk_used"] = data["disk_total"] - data["disk_available"]
-        data["host_memory_total"] = summary.hardware.memorySize / (1024 * 1024)
-        data["host_memory_free"] = data["host_memory_total"] -\
-                                   summary.quickStats.overallMemoryUsage
-        data["hypervisor_type"] = summary.config.product.name
-        data["hypervisor_version"] = summary.config.product.version
+        data["host_memory_total"] = stats['mem']['total']
+        data["host_memory_free"] = stats['mem']['free']
+        data["hypervisor_type"] = about_info.name
+        data["hypervisor_version"] = about_info.version
         data["hypervisor_hostname"] = self._host_name
         data["supported_instances"] = [('i686', 'vmware', 'hvm'),
                                        ('x86_64', 'vmware', 'hvm')]
