@@ -1,4 +1,6 @@
 #!/usr/bin/env python
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
 # Copyright (c) 2012 OpenStack Foundation
 # All Rights Reserved.
 #
@@ -17,25 +19,34 @@
 """Eventlet WSGI Services to proxy VNC for XCP protocol."""
 
 import socket
+import webob
 
 import eventlet
 import eventlet.green
 import eventlet.greenio
 import eventlet.wsgi
-from oslo_log import log as logging
-import webob
+from oslo.config import cfg
 
-import nova.conf
+from nova.consoleauth import rpcapi as consoleauth_rpcapi
 from nova import context
-from nova import exception
-from nova import objects
-from nova import utils
+from nova.openstack.common import log as logging
 from nova import version
 from nova import wsgi
 
 
 LOG = logging.getLogger(__name__)
-CONF = nova.conf.CONF
+
+xvp_proxy_opts = [
+    cfg.IntOpt('xvpvncproxy_port',
+               default=6081,
+               help='Port that the XCP VNC proxy should bind to'),
+    cfg.StrOpt('xvpvncproxy_host',
+               default='0.0.0.0',
+               help='Address that the XCP VNC proxy should bind to'),
+    ]
+
+CONF = cfg.CONF
+CONF.register_opts(xvp_proxy_opts)
 
 
 class XCPVNCProxy(object):
@@ -46,7 +57,7 @@ class XCPVNCProxy(object):
         while True:
             try:
                 d = source.recv(32384)
-            except Exception:
+            except Exception as e:
                 d = None
 
             # If recv fails, send a write shutdown the other direction
@@ -57,22 +68,22 @@ class XCPVNCProxy(object):
             try:
                 # sendall raises an exception on write error, unlike send
                 dest.sendall(d)
-            except Exception:
+            except Exception as e:
                 source.close()
                 dest.close()
                 break
 
     def handshake(self, req, connect_info, sockets):
         """Execute hypervisor-specific vnc auth handshaking (if needed)."""
-        host = connect_info.host
-        port = connect_info.port
+        host = connect_info['host']
+        port = int(connect_info['port'])
 
         server = eventlet.connect((host, port))
 
         # Handshake as necessary
-        if 'internal_access_path' in connect_info:
-            path = connect_info.internal_access_path
-            server.sendall('CONNECT %s HTTP/1.1\r\n\r\n' % path)
+        if connect_info.get('internal_access_path'):
+            server.sendall("CONNECT %s HTTP/1.1\r\n\r\n" %
+                        connect_info['internal_access_path'])
 
             data = ""
             while True:
@@ -81,27 +92,28 @@ class XCPVNCProxy(object):
                     data += b
                     if data.find("\r\n\r\n") != -1:
                         if not data.split("\r\n")[0].find("200"):
-                            LOG.info("Error in handshake format: %s", data)
+                            LOG.audit(_("Error in handshake: %s"), data)
                             return
                         break
 
                 if not b or len(data) > 4096:
-                    LOG.info("Error in handshake: %s", data)
+                    LOG.audit(_("Error in handshake: %s"), data)
                     return
 
         client = req.environ['eventlet.input'].get_socket()
         client.sendall("HTTP/1.1 200 OK\r\n\r\n")
+        socketsserver = None
         sockets['client'] = client
         sockets['server'] = server
 
     def proxy_connection(self, req, connect_info, start_response):
         """Spawn bi-directional vnc proxy."""
         sockets = {}
-        t0 = utils.spawn(self.handshake, req, connect_info, sockets)
+        t0 = eventlet.spawn(self.handshake, req, connect_info, sockets)
         t0.wait()
 
         if not sockets.get('client') or not sockets.get('server'):
-            LOG.info("Invalid request: %s", req)
+            LOG.audit(_("Invalid request: %s"), req)
             start_response('400 Invalid Request',
                            [('content-type', 'text/html')])
             return "Invalid Request"
@@ -109,8 +121,8 @@ class XCPVNCProxy(object):
         client = sockets['client']
         server = sockets['server']
 
-        t1 = utils.spawn(self.one_way_proxy, client, server)
-        t2 = utils.spawn(self.one_way_proxy, server, client)
+        t1 = eventlet.spawn(self.one_way_proxy, client, server)
+        t2 = eventlet.spawn(self.one_way_proxy, server, client)
         t1.wait()
         t2.wait()
 
@@ -121,27 +133,27 @@ class XCPVNCProxy(object):
     def __call__(self, environ, start_response):
         try:
             req = webob.Request(environ)
-            LOG.info("Request: %s", req)
+            LOG.audit(_("Request: %s"), req)
             token = req.params.get('token')
             if not token:
-                LOG.info("Request made with missing token: %s", req)
+                LOG.audit(_("Request made with missing token: %s"), req)
                 start_response('400 Invalid Request',
                                [('content-type', 'text/html')])
                 return "Invalid Request"
 
             ctxt = context.get_admin_context()
+            api = consoleauth_rpcapi.ConsoleAuthAPI()
+            connect_info = api.check_token(ctxt, token)
 
-            try:
-                connect_info = objects.ConsoleAuthToken.validate(ctxt, token)
-            except exception.InvalidToken:
-                LOG.info("Request made with invalid token: %s", req)
+            if not connect_info:
+                LOG.audit(_("Request made with invalid token: %s"), req)
                 start_response('401 Not Authorized',
                                [('content-type', 'text/html')])
                 return "Not Authorized"
 
             return self.proxy_connection(req, connect_info, start_response)
         except Exception as e:
-            LOG.info("Unexpected error: %s", e)
+            LOG.audit(_("Unexpected error: %s"), e)
 
 
 class SafeHttpProtocol(eventlet.wsgi.HttpProtocol):
@@ -161,15 +173,11 @@ class SafeHttpProtocol(eventlet.wsgi.HttpProtocol):
 
 
 def get_wsgi_server():
-    LOG.info("Starting nova-xvpvncproxy node (version %s)",
-             version.version_string_with_package())
-
-    LOG.warning('The nova-xvpvncproxy service is deprecated as it is Xen '
-                'specific and has effectively been replaced by noVNC '
-                'and the nova-novncproxy service.')
+    LOG.audit(_("Starting nova-xvpvncproxy node (version %s)"),
+              version.version_string_with_package())
 
     return wsgi.Server("XCP VNC Proxy",
                        XCPVNCProxy(),
                        protocol=SafeHttpProtocol,
-                       host=CONF.vnc.xvpvncproxy_host,
-                       port=CONF.vnc.xvpvncproxy_port)
+                       host=CONF.xvpvncproxy_host,
+                       port=CONF.xvpvncproxy_port)
